@@ -6,7 +6,7 @@ from eomaps.callbacks import (
 )
 from types import SimpleNamespace
 
-from functools import update_wrapper, partial, wraps, lru_cache
+from functools import update_wrapper, partial, wraps
 from collections import defaultdict
 import matplotlib.pyplot as plt
 
@@ -53,7 +53,9 @@ class _cb_container(object):
                 event = self._event
 
             for m in [*self._m.parent._children, self._m.parent]:
-                if event.inaxes is m.figure.ax:
+                # don't use "is" in here since Maps-children are proxies
+                # (and so are their attributes)!
+                if event.inaxes == m.figure.ax:
                     obj = self._getobj(m)
                     if obj is not None:
                         objs.append(obj)
@@ -117,6 +119,10 @@ class _click_container(_cb_container):
     def __init__(self, m, cb_cls=None, method="pick"):
         super().__init__(m, cb_cls, method)
 
+        # a dict to identify connected _move callbacks
+        # (e.g. to remove "_move" and "click" cbs in one go)
+        self._connected_move_cbs = dict()
+
     class _attach:
         """
         Attach custom or pre-defined callbacks to the map.
@@ -154,7 +160,7 @@ class _click_container(_cb_container):
             >>>     print("the position of the clicked pixel", kwargs["pos"])
             >>>     print("the data-index of the clicked pixel", kwargs["ID"])
             >>>     print("data-value of the clicked pixel", kwargs["val"])
-            >>>     print("the plot-crs is:", self.plot_specs["plot_crs"])
+            >>>     print("the plot-crs is:", self.crs_plot)
 
             >>> m.cb.pick.attach(some_callback, double_click=False, button=1, asdf=1)
         """
@@ -183,15 +189,13 @@ class _click_container(_cb_container):
                 the function to attach to the map.
                 The call-signature is:
 
-                >>> def some_callback(self, **kwargs):
+                >>> def some_callback(asdf, **kwargs):
                 >>>     print("hello world")
                 >>>     print("the position of the clicked pixel", kwargs["pos"])
                 >>>     print("the data-index of the clicked pixel", kwargs["ID"])
                 >>>     print("data-value of the clicked pixel", kwargs["val"])
-                >>>     print("the plot-crs is:", self.plot_specs["plot_crs"])
                 >>>
-                >>> m.cb.attach(some_callback)
-
+                >>> m.cb.attach(some_callback, asdf=1)
 
             double_click : bool
                 Indicator if the callback should be executed on double-click (True)
@@ -208,7 +212,6 @@ class _click_container(_cb_container):
                 kwargs passed to the callback-function
                 For documentation of the individual functions check the docs in `m.cb`
 
-
             Returns
             -------
             cid : int
@@ -279,11 +282,19 @@ class _click_container(_cb_container):
         ----------
         callback : int, str or tuple
             if str: the name of the callback to remove
-                    (`<function_name>_<count>__<double/single>__<button_ID>`)
+                    (`<function_name>_<count>__<layer>__<double/single>__<button_ID>`)
         """
+        # remove motion callbacks connected to click-callbacks
+        if self._method == "click":
+            if callback in self._connected_move_cbs:
+                for i in self._connected_move_cbs[callback]:
+                    self._m.cb._move.remove(i)
+
         if callback is not None:
             s = callback.split("__")
-            name, ds, b = s
+            name, layer, ds, b = s
+
+        cbname = name + "__" + layer
 
         dsdict = self.get.cbs.get(ds, None)
         if dsdict is not None:
@@ -297,8 +308,8 @@ class _click_container(_cb_container):
             return
 
         if bdict is not None:
-            if name in bdict:
-                del bdict[name]
+            if cbname in bdict:
+                del bdict[cbname]
 
                 # call cleanup methods on removal
                 fname = name.rsplit("_", 1)[0]
@@ -338,13 +349,12 @@ class _click_container(_cb_container):
                 - "clear_markers" : clear all existing markers
 
             You can also define a custom function with the following call-signature:
-                >>> def some_callback(self, asdf, **kwargs):
+                >>> def some_callback(asdf, **kwargs):
                 >>>     print("hello world")
                 >>>     print("the position of the clicked pixel", kwargs["pos"])
                 >>>     print("the data-index of the clicked pixel", kwargs["ID"])
                 >>>     print("data-value of the clicked pixel", kwargs["val"])
-                >>>     print("the plot-crs is:", self.m.plot_specs["plot_crs"])
-                >>>     print("asdf is:", asdf)
+                >>>     print("asdf is set to:", asdf)
 
                 >>> m.cb.attach(some_callback, double_click=False, button=1, asdf=1)
 
@@ -359,6 +369,14 @@ class _click_container(_cb_container):
                 - RIGHT = 3
                 - BACK = 8
                 - FORWARD = 9
+        on_motion : bool
+            !! Only relevant for "click" callbacks !!
+
+            - True: Continuously execute the callback if the mouse is moved while the
+              assigned button is pressed.
+            - False: Only execute the callback on clicks.
+
+            The default is True.
         **kwargs :
             kwargs passed to the callback-function
             For documentation of the individual functions check the docs in `m.cb`
@@ -377,6 +395,21 @@ class _click_container(_cb_container):
                 + "... use `m.plot_map()` first."
             )
 
+        # attach "on_move" callbacks
+        movecb_name = None
+        # set on_motion True for "click" callbacks and False otherwise
+        on_motion = kwargs.pop("on_motion", True if self._method == "click" else False)
+        if self._method == "click" and on_motion is True:
+            movecb_name = self._m.cb._move._add_callback(
+                *args,
+                callback=callback,
+                double_click=double_click,
+                button=button,
+                **kwargs,
+            )
+        elif on_motion is True:
+            print("EOmaps: 'on_motion=True' is only possible for 'click' callbacks!")
+
         assert not all(
             i in kwargs for i in ["pos", "ID", "val", "double_click", "button"]
         ), 'the names "pos", "ID", "val" cannot be used as keyword-arguments!'
@@ -388,17 +421,6 @@ class _click_container(_cb_container):
                 + "\n    - ".join(self._cb_list)
             )
             callback = getattr(self._cb, callback)
-        elif callable(callback):
-            # re-bind the callback methods to the eomaps.Maps.cb object
-            # in case custom functions are used
-            if hasattr(callback, "__func__"):
-                callback = callback.__func__.__get__(self._m)
-            else:
-                callback = callback.__get__(self._m)
-
-        # make sure multiple callbacks of the same funciton are only assigned
-        # if multiple assignments are properly handled
-        multi_cb_functions = ["mark", "annotate"]
 
         if double_click is True:
             btn_key = "double"
@@ -410,19 +432,25 @@ class _click_container(_cb_container):
         d = self.get.cbs[btn_key][button]
 
         # get a unique name for the callback
-        ncb = [int(i.rsplit("_", 1)[1]) for i in d if i.startswith(callback.__name__)]
-        cbkey = callback.__name__ + f"_{max(ncb) + 1 if len(ncb) > 0 else 0}"
-
-        # if callback.__name__ not in multi_cb_functions:
-        #     assert len(ncb) == 0, (
-        #         f"Multiple assignments of the callback '{callback.__name__}' using "
-        #         + "the same button is not (yet) supported... use a different button!"
-        #     )
+        # name_idx__layer
+        ncb = [
+            int(i.split("__", 1)[0].rsplit("_", 1)[1])
+            for i in d
+            if i.startswith(callback.__name__)
+        ]
+        cbkey = (
+            callback.__name__
+            + f"_{max(ncb) + 1 if len(ncb) > 0 else 0}"
+            + f"__{self._m.layer}"
+        )
 
         d[cbkey] = partial(callback, *args, **kwargs)
 
         # add mouse-button assignment as suffix to the name (with __ separator)
-        cbname = cbkey + f"__{btn_key}__{button}"
+        cbname = cbkey + f"__{btn_key}__{button}"  # TODO
+
+        if movecb_name is not None:
+            self._connected_move_cbs[cbname] = [movecb_name]
 
         return cbname
 
@@ -454,7 +482,6 @@ class cb_click_container(_click_container):
         self._cid_button_press_event = None
         self._cid_button_release_event = None
         self._cid_motion_event = None
-        self._only = []
 
     def _init_cbs(self):
         if self._m.parent is self._m:
@@ -481,10 +508,12 @@ class cb_click_container(_click_container):
         if event.button in cbs:
             bcbs = cbs[event.button]
             for key in self._sort_cbs(bcbs):
+                layer = key.split("__")[1]
+                if layer != "all" and layer != str(self._m.BM.bg_layer):
+                    return
+
                 cb = bcbs[key]
-                if clickdict is not None and (
-                    len(self._only) == 0 or key in self._only
-                ):
+                if clickdict is not None:
                     cb(**clickdict)
 
     def _onrelease(self, event):
@@ -496,76 +525,65 @@ class cb_click_container(_click_container):
                 cb()
 
     def _reset_cids(self):
+        if self._cid_button_press_event:
+            self._m.figure.f.canvas.mpl_disconnect(self._cid_button_press_event)
         self._cid_button_press_event = None
+
+        if self._cid_motion_event:
+            self._m.figure.f.canvas.mpl_disconnect(self._cid_motion_event)
         self._cid_motion_event = None
 
+        if self._cid_button_release_event:
+            self._m.figure.f.canvas.mpl_disconnect(self._cid_button_release_event)
+        self._cid_button_release_event = None
+
     def _add_click_callback(self):
         def clickcb(event):
-            self._event = event
+            try:
+                self._event = event
 
-            # ignore callbacks while dragging axes
-            if self._m._ignore_cb_events:
-                return
-            # don't execute callbacks if a toolbar-action is active
-            if (
-                self._m.figure.f.canvas.toolbar is not None
-            ) and self._m.figure.f.canvas.toolbar.mode != "":
-                return
+                # ignore callbacks while dragging axes
+                if self._m._ignore_cb_events:
+                    return
+                # don't execute callbacks if a toolbar-action is active
+                if (
+                    self._m.figure.f.canvas.toolbar is not None
+                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    return
 
-            # execute onclick on the maps object that belongs to the clicked axis
-            # and forward the event to all forwarded maps-objects
-            for obj in self._objs:
-                obj._onclick(event)
-                obj._m.BM._after_update_actions.append(obj._clear_temporary_artists)
-                # forward callbacks to the connected maps-objects
-                obj._fwd_cb(event)
+                # execute onclick on the maps object that belongs to the clicked axis
+                # and forward the event to all forwarded maps-objects
 
-            self._m.parent.BM.update(clear=self._method)
+                for obj in self._objs:
+                    obj._onclick(event)
+                    obj._m.BM._after_update_actions.append(obj._clear_temporary_artists)
+                    # forward callbacks to the connected maps-objects
+                    obj._fwd_cb(event)
 
-        def movecb(event):
-            self._event = event
-
-            # ignore callbacks while dragging axes
-            if self._m._ignore_cb_events:
-                return
-            # don't execute callbacks if a toolbar-action is active
-            if (
-                self._m.figure.f.canvas.toolbar is not None
-            ) and self._m.figure.f.canvas.toolbar.mode != "":
-                return
-
-            # only execute movecb if a mouse-button is holded down
-            # and only if the motion is happening inside the axes
-            if not event.button:  # or (event.inaxes != self._m.figure.ax):
-                return
-
-            # execute onclick on the maps object that belongs to the clicked axis
-            # and forward the event to all forwarded maps-objects
-            for obj in self._objs:
-                obj._onclick(event)
-                obj._m.BM._after_update_actions.append(obj._clear_temporary_artists)
-                # forward callbacks to the connected maps-objects
-                obj._fwd_cb(event)
-
-            self._m.parent.BM.update(clear=self._method)
+                self._m.parent.BM.update(clear=self._method)
+            except ReferenceError:
+                pass
 
         def releasecb(event):
-            # ignore callbacks while dragging axes
-            if self._m._ignore_cb_events:
-                return
-            # don't execute callbacks if a toolbar-action is active
-            if (
-                self._m.figure.f.canvas.toolbar is not None
-            ) and self._m.figure.f.canvas.toolbar.mode != "":
-                return
-
-            # execute onclick on the maps object that belongs to the clicked axis
-            # and forward the event to all forwarded maps-objects
-            for obj in self._objs:
-                obj._onrelease(event)
-                # forward callbacks to the connected maps-objects
-                obj._fwd_cb(event)
+            try:
+                # ignore callbacks while dragging axes
+                if self._m._ignore_cb_events:
+                    return
+                # don't execute callbacks if a toolbar-action is active
+                if (
+                    self._m.figure.f.canvas.toolbar is not None
+                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    return
 
+                # execute onclick on the maps object that belongs to the clicked axis
+                # and forward the event to all forwarded maps-objects
+                for obj in self._objs:
+                    obj._onrelease(event)
+                    # forward callbacks to the connected maps-objects
+                    obj._fwd_cb(event)
+            except ReferenceError:
+                # ignore errors caused by no-longer existing weakrefs
+                pass
             # self._m.parent.BM.update(clear=False)
 
         if self._cid_button_press_event is None:
@@ -580,12 +598,6 @@ class cb_click_container(_click_container):
                 "button_release_event", releasecb
             )
 
-        if self._cid_motion_event is None:
-            # for click-callbacks, allow motion-detection
-            self._cid_motion_event = self._m.figure.f.canvas.mpl_connect(
-                "motion_notify_event", movecb
-            )
-
     def _fwd_cb(self, event):
         # click container events are MouseEvents!
         if event.inaxes != self._m.figure.ax:
@@ -629,6 +641,84 @@ class cb_click_container(_click_container):
                 m.BM._after_update_actions.append(obj._clear_temporary_artists)
 
 
+class cb_move_container(cb_click_container):
+    """
+    Callbacks that are executed if you move the mouse while holding down a button.
+    (used internally to make click-callbacks motion-sensitive, but it can be
+     used just as any other callback with `m.cb._move.attach...`)
+
+
+    Methods
+    -------
+
+    attach : accessor for callbacks.
+        Executing the functions will attach the associated callback to the map!
+
+    get : accessor for return-objects
+        A container to provide easy-access to the return-values of the callbacks.
+
+    remove : remove prviously added callbacks from the map
+
+    forward_events : forward events to connected maps-objects
+
+    share_events : share events between connected maps-objects (e.g. forward both ways)
+
+    """
+
+    # this is just a copy of cb_click_container to manage motion-sensitive callbacks
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self._cid_motion_event = None
+
+    def _init_cbs(self):
+        if self._m.parent is self._m:
+            self._add_move_callback()
+
+    def _reset_cids(self):
+        if self._cid_motion_event:
+            self._m.figure.f.canvas.mpl_disconnect(self._cid_motion_event)
+        self._cid_motion_event = None
+
+    def _add_move_callback(self):
+        def movecb(event):
+            try:
+                self._event = event
+
+                # only execute movecb if a mouse-button is holded down
+                # and only if the motion is happening inside the axes
+                if not event.button:  # or (event.inaxes != self._m.figure.ax):
+                    return
+
+                # ignore callbacks while dragging axes
+                if self._m._ignore_cb_events:
+                    return
+                # don't execute callbacks if a toolbar-action is active
+                if (
+                    self._m.figure.f.canvas.toolbar is not None
+                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    return
+
+                # execute onclick on the maps object that belongs to the clicked axis
+                # and forward the event to all forwarded maps-objects
+                for obj in self._objs:
+                    obj._onclick(event)
+                    obj._m.BM._after_update_actions.append(obj._clear_temporary_artists)
+                    # forward callbacks to the connected maps-objects
+                    obj._fwd_cb(event)
+
+                self._m.parent.BM.update(clear=self._method)
+            except ReferenceError:
+                pass
+
+        if self._cid_motion_event is None:
+            # for click-callbacks, allow motion-detection
+            self._cid_motion_event = self._m.figure.f.canvas.mpl_connect(
+                "motion_notify_event", movecb
+            )
+
+
 class cb_pick_container(_click_container):
     """
     Callbacks that select the nearest datapoint if you click on the map.
@@ -694,8 +784,19 @@ class cb_pick_container(_click_container):
         self._add_pick_callback()
 
     def _default_picker(self, artist, event):
-        # if no pick-callback is attached, don't identify the picked point
-        if len(self._m.cb.pick.get.cbs) == 0:
+
+        # make sure that objects are only picked if we are on the right layer
+        if self._m.layer != self._m.BM.bg_layer:
+            return False, None
+
+        try:
+            # if no pick-callback is attached, don't identify the picked point
+            if len(self._m.cb.pick.get.cbs) == 0:
+                return False, None
+        except ReferenceError:
+            # in case we encounter a reference-error, remove the picker from the artist
+            # (happens if the picker originates from a no-longer existing Maps object)
+            self._artist.set_picker(None)
             return False, None
 
         if (event.inaxes != self._m.ax) or not hasattr(self._m, "tree"):
@@ -725,8 +826,11 @@ class cb_pick_container(_click_container):
         if ind is not None:
             if self._m.figure.coll is not None and event.artist is self._m.figure.coll:
                 clickdict = dict(
-                    pos=(self._m._props["x0"][ind], self._m._props["y0"][ind]),
-                    ID=self._m._props["ids"][ind],
+                    pos=(
+                        self._m._props["x0"].flat[ind],
+                        self._m._props["y0"].flat[ind],
+                    ),
+                    ID=self._m._props["ids"].flat[ind],
                     val=self._m._props["z_data"][ind],
                     ind=ind,
                     picker_name=self._picker_name,
@@ -747,6 +851,11 @@ class cb_pick_container(_click_container):
         if event.artist is not self._artist:
             return
 
+        # only execute onpick if the correct layer is visible
+        # (relevant for forwarded callbacks)
+        if self._m.layer != self._m.BM.bg_layer:
+            return
+
         # don't execute callbacks if a toolbar-action is active
         if (
             self._m.figure.f.canvas.toolbar is not None
@@ -754,6 +863,7 @@ class cb_pick_container(_click_container):
             return
 
         clickdict = self._get_pickdict(event)
+
         if event.mouseevent.dblclick:
             cbs = self.get.cbs["double"]
         else:
@@ -762,44 +872,62 @@ class cb_pick_container(_click_container):
         if event.mouseevent.button in cbs:
             bcbs = cbs[event.mouseevent.button]
             for key in self._sort_cbs(bcbs):
+                layer = key.split("__")[1]
+                if layer != "all" and layer != str(self._m.BM.bg_layer):
+                    # TODO
+                    # only execute callbacks if the layer name of the associated
+                    # maps-object is active
+                    return
+
                 cb = bcbs[key]
                 if clickdict is not None:
                     cb(**clickdict)
 
     def _reset_cids(self):
-        self._cid_pick_event = dict()
+        for method, cid in self._cid_pick_event.items():
+            self._m.figure.f.canvas.mpl_disconnect(cid)
+        self._cid_pick_event.clear()
 
     def _add_pick_callback(self):
         # execute onpick and forward the event to all connected Maps-objects
 
         def pickcb(event):
-            # check if we want to ignore callbacks
-            if self._m._ignore_cb_events:
-                return
+            try:
 
-            # don't execute callbacks if a toolbar-action is active
-            if (
-                self._m.figure.f.canvas.toolbar is not None
-            ) and self._m.figure.f.canvas.toolbar.mode != "":
-                return
+                # make sure pickcb is only executed if we are on the right layer
+                if self._m.layer != self._m.BM.bg_layer:
+                    return
 
-            if not self._artist is event.artist:
-                return
+                # check if we want to ignore callbacks
+                if self._m._ignore_cb_events:
+                    return
 
-            self._event = event
-            # check if the artists has a custom picker assigned
+                # don't execute callbacks if a toolbar-action is active
+                if (
+                    self._m.figure.f.canvas.toolbar is not None
+                ) and self._m.figure.f.canvas.toolbar.mode != "":
+                    return
 
-            # execute "_onpick" on the maps-object that belongs to the clicked axes
-            # and forward the event to all forwarded maps-objects
-            self._onpick(event)
-            # forward callbacks to the connected maps-objects
-            self._fwd_cb(event, self._picker_name)
+                if not self._artist is event.artist:
+                    return
 
-            self._m.BM._after_update_actions.append(self._clear_temporary_artists)
-            self._m.BM._clear_temp_artists(self._method)
+                self._event = event
+                # check if the artists has a custom picker assigned
 
-            # self._m.parent.BM.update(clear=self._method)
-            # don't update here... the click-callback will take care of it!
+                # execute "_onpick" on the maps-object that belongs to the clicked axes
+                # and forward the event to all forwarded maps-objects
+                self._onpick(event)
+                # forward callbacks to the connected maps-objects
+                self._fwd_cb(event, self._picker_name)
+
+                self._m.BM._after_update_actions.append(self._clear_temporary_artists)
+                self._m.BM._clear_temp_artists(self._method)
+
+                # self._m.parent.BM.update(clear=self._method)
+                self._m.parent.BM._clear_temp_artists(self._method)
+                # don't update here... the click-callback will take care of it!
+            except ReferenceError:
+                pass
 
         # attach the callbacks (only once per method!)
         if self._method not in self._cid_pick_event:
@@ -888,27 +1016,41 @@ class keypress_container(_cb_container):
     def __init__(self, m, cb_cls=None, method="keypress"):
         super().__init__(m, cb_cls, method)
 
+        self._cid_keypress_event = None
+
     def _init_cbs(self):
         if self._m.parent is self._m:
             self._initialize_callbacks()
 
     def _reset_cids(self):
+        if self._cid_keypress_event:
+            self._m.figure.f.canvas.mpl_disconnect(self._cid_keypress_event)
         self._cid_keypress_event = None
 
     def _initialize_callbacks(self):
         def _onpress(event):
-            self._event = event
+            try:
+                self._event = event
 
-            if event.key in self.get.cbs:
-                # do this to allow deleting callbacks with a callback
-                cbs = self.get.cbs[event.key]
-                names = list(cbs)
-                for name in names:
-                    if name in cbs:
-                        cbs[name](key=event.key)
+                for obj in self._objs:
+                    # only trigger callbacks on the right layer
+                    if (obj._m.layer != "all") and (
+                        obj._m.layer != self._m.BM.bg_layer
+                    ):
+                        continue
+                    if event.key in obj.get.cbs:
 
-                # for name, cb in self.get.cbs[event.key].items():
-                #     cb(key=event.key)
+                        # do this to allow deleting callbacks with a callback
+                        # otherwise modifying a dict during iteration is problematic!
+                        cbs = obj.get.cbs[event.key]
+                        names = list(cbs)
+                        for name in names:
+                            if name in cbs:
+                                cbs[name](key=event.key)
+
+                self._m.parent.BM.update(clear=self._method)
+            except ReferenceError:
+                pass
 
         if self._m is self._m.parent:
             self._cid_keypress_event = self._m.figure.f.canvas.mpl_connect(
@@ -958,10 +1100,10 @@ class keypress_container(_cb_container):
                 the function to attach to the map.
                 The call-signature is:
 
-                >>> def some_callback(self, **kwargs):
-                >>>     print("hello world")
+                >>> def some_callback(asdf, **kwargs):
+                >>>     print("hello world, asdf=", asdf)
                 >>>
-                >>> m.cb.attach(some_callback)
+                >>> m.cb.attach(some_callback, asdf=1)
 
             key : str
                 the key to use
@@ -1003,16 +1145,19 @@ class keypress_container(_cb_container):
         ----------
         callback : int, str or tuple
             if str: the name of the callback to remove
-                    (`<function_name>_<count>__<key>`)
+                    (`<function_name>_<count>__<layer>__<key>`)
         """
+
         if callback is not None:
-            name, key = callback.split("__")
+            name, layer, key = callback.split("__")
+
+        cbname = name + "__" + layer
 
         cbs = self.get.cbs.get(key, None)
 
         if cbs is not None:
-            if name in cbs:
-                del cbs[name]
+            if cbname in cbs:
+                del cbs[cbname]
 
                 # call cleanup methods on removal
                 fname = name.rsplit("_", 1)[0]
@@ -1063,20 +1208,17 @@ class keypress_container(_cb_container):
                 + "\n    - ".join(self._cb_list)
             )
             callback = getattr(self._cb, callback)
-        elif callable(callback):
-            # re-bind the callback methods to the eomaps.Maps.cb object
-            # in case custom functions are used
-            if hasattr(callback, "__func__"):
-                callback = callback.__func__.__get__(self._m)
-            else:
-                callback = callback.__get__(self._m)
 
         cbdict = self.get.cbs[key]
         # get a unique name for the callback
         ncb = [
             int(i.rsplit("_", 1)[1]) for i in cbdict if i.startswith(callback.__name__)
         ]
-        cbkey = callback.__name__ + f"_{max(ncb) + 1 if len(ncb) > 0 else 0}"
+        cbkey = (
+            callback.__name__
+            + f"_{max(ncb) + 1 if len(ncb) > 0 else 0}"
+            + f"__{self._m.layer}"
+        )
 
         # append the callback
         cbdict[cbkey] = partial(callback, **kwargs)
@@ -1105,43 +1247,58 @@ class cb_container:
     def __init__(self, m):
         self._m = m
 
-        self._methods = ["click", "keypress"]
+        self._methods = ["click", "move", "keypress"]
 
-    @property
-    @lru_cache()
-    @wraps(cb_click_container)
-    def click(self):
-        return cb_click_container(
+        self._click = cb_click_container(
             m=self._m,
             cb_cls=click_callbacks,
             method="click",
         )
 
-    @property
-    @lru_cache()
-    @wraps(cb_pick_container)
-    def pick(self):
-        return cb_pick_container(
+        self._move = cb_move_container(
+            m=self._m,
+            cb_cls=click_callbacks,
+            method="move",
+        )
+
+        self._pick = cb_pick_container(
             m=self._m,
             cb_cls=pick_callbacks,
             method="pick",
         )
 
-    @property
-    @lru_cache()
-    @wraps(keypress_container)
-    def keypress(self):
-        return keypress_container(
+        self._keypress = keypress_container(
             m=self._m,
             cb_cls=keypress_callbacks,
             method="keypress",
         )
 
+        self._dynamic = dynamic_callbacks(m=self._m)
+
+    @property
+    @wraps(cb_click_container)
+    def click(self):
+        return self._click
+
+    @property
+    @wraps(cb_move_container)
+    def move(self):
+        return self._move
+
+    @property
+    @wraps(cb_pick_container)
+    def pick(self):
+        return self._pick
+
+    @property
+    @wraps(keypress_container)
+    def keypress(self):
+        return self._keypress
+
     @property
-    @lru_cache()
     @wraps(dynamic_callbacks)
     def dynamic(self):
-        return dynamic_callbacks(m=self._m)
+        return self._dynamic
 
     def add_picker(self, name, artist, picker):
         """
