@@ -104,8 +104,7 @@ class map_objects(object):
     # @wraps(plt.Axes.set_position)
     def set_colorbar_position(self, pos=None, ratio=None, cb=None):
         """
-        a wrapper to set the position of the colorbar and the histogram at
-        the same time
+        Set the position (and ratio) of a previously created colorbar with a histogram.
 
         Parameters
         ----------
@@ -120,19 +119,30 @@ class map_objects(object):
         cb : list, optional
             The colorbar-objects (as returned by `m.add_colorbar()`)
             If None, the existing colorbar will be used.
+
+        Examples
+        --------
+
+        Change both the position and the ratio between colorbar and histogram
+
+        >>> m.figure.set_colorbar_position(pos=[.1, .1, .8, .2], ratio=15)
+
+        Change only the ratio between colorbar and histogram
+
+        >>> m.figure.set_colorbar_position(ratio=999)
+
+        Change only the position of colorbar and histogram
+
+        >>> m.figure.set_colorbar_position(pos=[.1, .1, .8, .2])
+
         """
 
         if cb is None:
-            _, ax_cb, ax_cb_plot, orientation = [
-                self.cb_gridspec,
-                self.ax_cb,
-                self.ax_cb_plot,
-                self._m._orientation,
-            ]
+            _, _, ax_cb, ax_cb_plot, orientation, _ = self._m._colorbar
         else:
-            _, ax_cb, ax_cb_plot, orientation, _ = cb
+            _, _, ax_cb, ax_cb_plot, orientation, _ = cb
 
-        if orientation == "vertical":
+        if orientation == "horizontal":
             pcb = ax_cb.get_position()
             pcbp = ax_cb_plot.get_position()
             if pos is None:
@@ -150,7 +160,7 @@ class map_objects(object):
                 [pos[0], pos[1] + hcb, pos[2], hp],
             )
 
-        elif orientation == "horizontal":
+        elif orientation == "vertical":
             pcb = ax_cb.get_position()
             pcbp = ax_cb_plot.get_position()
             if pos is None:
@@ -170,6 +180,9 @@ class map_objects(object):
         else:
             raise TypeError(f"EOmaps: '{orientation}' is not a valid orientation")
 
+        # re-fetch the background layer to make changes visible
+        self._m.BM.fetch_bg(self._m.layer)
+
 
 class data_specs(object):
     """
@@ -180,37 +193,64 @@ class data_specs(object):
         self,
         m,
         data=None,
-        xcoord="lon",
-        ycoord="lat",
+        x="lon",
+        y="lat",
         crs=4326,
         parameter=None,
+        encoding=None,
+        cpos="c",
+        cpos_radius=None,
+        **kwargs,
     ):
         self._m = m
         self.data = data
-        self.xcoord = xcoord
-        self.ycoord = ycoord
+        self.x = x
+        self.y = y
         self.crs = crs
         self.parameter = parameter
 
+        self._encoding = encoding
+        self._cpos = cpos
+        self._cpos_radius = cpos_radius
+
     def delete(self):
         self._data = None
-        self._xcoord = None
-        self._ycoord = None
+        self._x = None
+        self._y = None
         self._crs = None
         self._parameter = None
+        self._encoding = False
+        self._cpos = "c"
+        self._cpos_radius = False
 
     def __repr__(self):
-        txt = f"""\
-              # parameter = {self.parameter}
-              # coordinates = ({self.xcoord}, {self.ycoord})
-              # crs: {indent(fill(self.crs.__repr__(), 60),
-                              "                      ").strip()}
+        try:
+            txt = f"""\
+                  # parameter: {self.parameter}
+                  # x: {indent(fill(self.x.__repr__(), 60),
+                                    "                      ").strip()}
+                  # y: {indent(fill(self.y.__repr__(), 60),
+                                    "                      ").strip()}
 
-              # data:\
-              {indent(self.data.__repr__(), "                ")}
-              """
+                  # crs: {indent(fill(self.crs.__repr__(), 60),
+                                 "                      ").strip()}
 
-        return dedent(txt)
+                  # data: {indent(self.data.__repr__(),
+                                  "                ").lstrip()}
+                  """
+            if self.encoding:
+                txt += dedent(
+                    f"""\
+                    # encoding: {indent(fill(self.encoding.__repr__(), 60),
+                    "                ").lstrip()}
+                    """
+                )
+            if self.cpos_radius:
+                txt += f"# cpos: {'self.cpos'} (cpos_radius={self.cpos_radius})"
+
+            return dedent(txt)
+        except:
+            return object.__repr__(self)
 
     def __getitem__(self, key):
         if isinstance(key, (list, tuple)):
@@ -253,12 +293,25 @@ class data_specs(object):
         if key == "crs":
             key = "in_crs"
 
-        assert key in self.keys(), f"{key} is not a valid data-specs key!"
+        assert key in [
+            *self.keys(),
+            "xcoord",
+            "ycoord",
+        ], f"{key} is not a valid data-specs key!"
 
         return key
 
     def keys(self):
-        return ("parameter", "xcoord", "ycoord", "in_crs", "data")
+        return (
+            "parameter",
+            "x",
+            "y",
+            "in_crs",
+            "data",
+            "encoding",
+            "cpos",
+            "cpos_radius",
+        )
 
     @property
     def data(self):
@@ -280,19 +333,59 @@ class data_specs(object):
 
     @property
     def xcoord(self):
-        return self._xcoord
+        warn(
+            "EOmaps: `m.data_specs.xcoord` is depreciated."
+            + "use `m.data_specs.x` instead!",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self._x
 
     @xcoord.setter
     def xcoord(self, xcoord):
-        self._xcoord = xcoord
+        warn(
+            "EOmaps: `m.data_specs.xcoord` is depreciated."
+            + "use `m.data_specs.x` instead!",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        self._x = xcoord
 
     @property
     def ycoord(self):
-        return self._ycoord
+        warn(
+            "EOmaps: `m.data_specs.ycoord` is depreciated."
+            + "use `m.data_specs.y` instead!",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self._y
 
     @ycoord.setter
     def ycoord(self, ycoord):
-        self._ycoord = ycoord
+        warn(
+            "EOmaps: `m.data_specs.ycoord` is depreciated."
+            + "use `m.data_specs.y` instead!",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        self._y = ycoord
+
+    @property
+    def x(self):
+        return self._x
+
+    @x.setter
+    def x(self, x):
+        self._x = x
+
+    @property
+    def y(self):
+        return self._y
+
+    @y.setter
+    def y(self, y):
+        self._y = y
 
     @property
     def parameter(self):
@@ -305,17 +398,11 @@ class data_specs(object):
     @parameter.getter
     def parameter(self):
         if _pd_OK and isinstance(self.data, pd.DataFrame) and self._parameter is None:
-            if (
-                self.data is not None
-                and self.xcoord is not None
-                and self.ycoord is not None
-            ):
+            if self.data is not None and self.x is not None and self.y is not None:
 
                 try:
                     self.parameter = next(
-                        i
-                        for i in self.data.keys()
-                        if i not in [self.xcoord, self.ycoord]
+                        i for i in self.data.keys() if i not in [self.x, self.y]
                     )
                     print(f"EOmaps: Parameter was set to: '{self.parameter}'")
 
@@ -327,99 +414,36 @@ class data_specs(object):
 
         return self._parameter
 
+    @property
+    def encoding(self):
+        return self._encoding
 
-class plot_specs(object):
-    """
-    a container for accessing the plot specifications
-    """
+    @encoding.setter
+    def encoding(self, encoding):
+        if encoding not in [None, False]:
+            assert isinstance(encoding, dict), "EOmaps: encoding must be a dictionary!"
 
-    def __init__(self, m, **kwargs):
-        self._m = m
+            assert all(
+                i in ["scale_factor", "add_offset"] for i in encoding
+            ), "EOmaps: encoding accepts only 'scale_factor' and 'add_offset' as keys!"
 
-        for key in kwargs:
-            assert key in self.keys(), f"'{key}' is not a valid data-specs key"
-
-            setattr(self, key, kwargs.get(key, None))
-
-    def __repr__(self):
-        txt = "\n".join(
-            f"# {key}: {indent(fill(self[key].__repr__(), 60),  ' '*(len(key) + 4)).strip()}"
-            for key in self.keys()
-        )
-        return txt
-
-    def __getitem__(self, key):
-        if isinstance(key, (list, tuple)):
-            for i in key:
-                assert i in self.keys(), f"{i} is not a valid plot-specs key!"
-            if len(key) == 0:
-                item = dict()
-            else:
-                key = list(key)
-                if len(key) == 1:
-                    item = {key[0]: getattr(self, key[0])}
-                else:
-                    item = dict(zip(key, attrgetter(*key)(self)))
-        else:
-            assert key in self.keys(), f"'{key}' is not a valid plot-specs key!"
-            item = getattr(self, key)
-        return item
-
-    def __setitem__(self, key, val):
-        key = self._sanitize_keys(key)
-        if key is not None:
-            return setattr(self, key, val)
-
-    def __setattr__(self, key, val):
-        key = self._sanitize_keys(key)
-        if key is not None:
-            super().__setattr__(key, val)
-
-    def __iter__(self):
-        return iter(self[self.keys()].items())
-
-    def _sanitize_keys(self, key):
-        # pass any keys starting with _
-        if key.startswith("_"):
-            return key
-
-        if key in ["crs", "plot_crs"]:
-            warn(
-                "\n▲▲▲ In EOmaps > v3.0 the plot-crs is set on "
-                + "initialization of the Maps-object!"
-                + "\n▲▲▲ Use `m = Maps(crs=...)` instead to set the plot-crs!\n"
-            )
-            return None
-
-        if key in ["title"]:
-            warn(
-                "\n▲▲▲ In EOmaps > v3.1 passing a 'title' to the plot-specs is depreciated. "
-                + "\n▲▲▲ Use `m.ax.set_title()` instead!\n"
-            )
-            return None
-
-        assert key in self.keys(), f"{key} is not a valid plot-specs key!"
-
-        return key
-
-    def keys(self):
-        # fmt: off
-        return ('label', 'cmap', 'histbins', 'tick_precision',
-                'vmin', 'vmax', 'cpos', 'cpos_radius', 'alpha', 'density')
-        # fmt: on
+        self._encoding = encoding
 
     @property
-    def cmap(self):
-        return self._cmap
+    def cpos(self):
+        return self._cpos
 
-    @cmap.setter
-    def cmap(self, val):
-        self._cmap = get_cmap(val)
+    @cpos.setter
+    def cpos(self, cpos):
+        self._cpos = cpos
 
     @property
-    @lru_cache()
-    def plot_crs(self):
-        return self._m._crs_plot
+    def cpos_radius(self):
+        return self._cpos_radius
+
+    @cpos_radius.setter
+    def cpos_radius(self, cpos_radius):
+        self._cpos_radius = cpos_radius
 
 
 class classify_specs(object):
@@ -559,7 +583,12 @@ class _NaturalEarth_presets:
         - fc="none", ec="k", zorder=100
         """
         return self._feature(
-            self._m, "physical", "coastline", fc="none", ec="k", zorder=100
+            self._m,
+            "physical",
+            "coastline",
+            fc="none",
+            ec="k",
+            zorder=100,
         )
 
     @property
@@ -1340,7 +1369,9 @@ else:
                     self._m = m
 
                     self.default = _xyz_tile_service(
-                        self._m, "https://tile.openstreetmap.org/{z}/{x}/{y}.png"
+                        self._m,
+                        "https://tile.openstreetmap.org/{z}/{x}/{y}.png",
+                        name="OSM_default",
                     )
                     self.default.__doc__ = combdoc(
                         """
@@ -1357,7 +1388,9 @@ else:
                     )
 
                     self.default_german = _xyz_tile_service(
-                        self._m, "https://tile.openstreetmap.de/{z}/{x}/{y}.png"
+                        self._m,
+                        "https://tile.openstreetmap.de/{z}/{x}/{y}.png",
+                        name="OSM_default_german",
                     )
                     self.default_german.__doc__ = combdoc(
                         """
@@ -1377,6 +1410,7 @@ else:
                         m=self._m,
                         url="https://c.tile.opentopomap.org/{z}/{x}/{y}.png",
                         maxzoom=16,
+                        name="OSM_OpenTopoMap",
                     )
                     self.OpenTopoMap.__doc__ = combdoc(
                         """
@@ -1397,48 +1431,60 @@ else:
                     self.stamen_toner = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner",
                     )
                     self.stamen_toner_lines = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner-lines/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner_lines",
                     )
                     self.stamen_toner_background = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner-background/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner_background",
                     )
                     self.stamen_toner_lite = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner-lite/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner_lite",
                     )
                     self.stamen_toner_hybrid = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner-hybrid/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner_hybrid",
                     )
                     self.stamen_toner_labels = _xyz_tile_service(
                         self._m,
                         "https://stamen-tiles.a.ssl.fastly.net/toner-labels/{z}/{x}/{y}.png",
+                        name="OSM_stamen_toner_labels",
                     )
 
                     self.stamen_watercolor = _xyz_tile_service(
                         self._m,
                         "http://c.tile.stamen.com/watercolor/{z}/{x}/{y}.jpg",
+                        name="OSM_stamen_watercolor",
                         maxzoom=18,
                     )
 
                     self.stamen_terrain = _xyz_tile_service(
-                        self._m, "http://c.tile.stamen.com/terrain/{z}/{x}/{y}.jpg"
+                        self._m,
+                        "http://c.tile.stamen.com/terrain/{z}/{x}/{y}.jpg",
+                        name="OSM_stamen_terrain",
                     )
                     self.stamen_terrain_lines = _xyz_tile_service(
                         self._m,
                         "http://c.tile.stamen.com/terrain-lines/{z}/{x}/{y}.jpg",
+                        name="OSM_stamen_terrain_lines",
                     )
                     self.stamen_terrain_labels = _xyz_tile_service(
                         self._m,
                         "http://c.tile.stamen.com/terrain-labels/{z}/{x}/{y}.jpg",
+                        name="OSM_stamen_terrain_labels",
                     )
                     self.stamen_terrain_background = _xyz_tile_service(
                         self._m,
                         "http://c.tile.stamen.com/terrain-background/{z}/{x}/{y}.jpg",
+                        name="OSM_stamen_terrain_background",
                     )
 
                     stamen_doc = """
@@ -1933,6 +1979,7 @@ else:
                     self._m,
                     lambda x, y, z: f"https://s1map.eodc.eu/vv/{z}/{x}/{2**z-1-y}.png",
                     13,
+                    name="S1GBM_vv",
                 )
 
                 WMS.__doc__ = combdoc("Polarization: VV", type(self).__doc__)
@@ -1944,10 +1991,49 @@ else:
                     self._m,
                     lambda x, y, z: f"https://s1map.eodc.eu/vh/{z}/{x}/{2**z-1-y}.png",
                     13,
+                    name="S1GBM_vh",
                 )
                 WMS.__doc__ = combdoc("Polarization: VH", type(self).__doc__)
                 return WMS
 
+        @property
+        @lru_cache()
+        def S2_cloudless(self):
+            """
+            Global cloudless Sentinel-2 maps, crafted by EOX.
+            https://s2maps.eu/
+
+            Endless sunshine, eternal summer - the Sentinel-2 cloudless layer combines
+            trillions of pixels collected during differing weather conditions during
+            the whole year of 2020 and merges them into a sunny homogeneous mosaic,
+            almost free from satellite and atmospheric effects. Our thanks go to the
+            European Commission and the European Space Agency for the free, full,
+            and open Sentinel-2 data.
+
+            Note
+            ----
+            **LICENSE-info (without any warranty for correctness!!)**
+
+            You are free to use Sentinel-2 cloudless as long as you follow the
+            applicable license conditions. The conditions for use are the attribution
+            when publishing any imagery or content from Sentinel-2 cloudless as well as
+            the non-commercial use for the 2018 and 2019 data. The attribution shall be
+            displayed legibly and in proximity to the usage, in on-line publications
+            (social-networks etc.) it shall include the links and show the text as
+            described below.
+
+            (check: https://s2maps.eu/ for full details)
+
+            """
+            WMS = _WebServiec_collection(
+                m=self._m,
+                service_type="wms",
+                url="https://tiles.maps.eox.at/wms?service=wms&request=getcapabilities",
+            )
+
+            WMS.__doc__ = type(self).S2_cloudless.__doc__
+            return WMS
+
         @property
         @lru_cache()
         def ESRI_ArcGIS(self):
