@@ -1,6 +1,7 @@
 from functools import wraps
 import numpy as np
 from pyproj import CRS
+from pathlib import Path
 
 try:
     import pandas as pd
@@ -39,7 +40,13 @@ class read_file:
 
     @staticmethod
     def GeoTIFF(
-        path_or_dataset, crs_key=None, data_crs=None, sel=None, isel=None, set_data=None
+        path_or_dataset,
+        crs_key=None,
+        data_crs=None,
+        sel=None,
+        isel=None,
+        set_data=None,
+        mask_and_scale=False,
     ):
         """
         Read all relevant information necessary to add a GeoTIFF to the map.
@@ -72,6 +79,24 @@ class read_file:
             Indicator if the dataset should be returned None or assigned to the
             provided Maps-object  (e.g. by using m.set_data(...)).
             The default is None.
+        mask_and_scale : bool
+            Indicator if the data should be masked and scaled with respect to the
+            file-attributes *_FillValue*, *scale_factor* and *add_offset*.
+
+            - If False: the data will only be scaled "on demand", avoiding costly
+              initialization of very large float-arrays. Masking will still be applied
+              in case a *_FillValue* attribute has been found.
+              (a masked-array is returned to avoid dtype conversions)
+              The encoding is accessible via `m.data_specs.encoding`
+            - If True: all data will be masked and scaled after reading.
+              For more details see `xarray.open_dataset`.
+
+            NOTE: using `mask_and_scale=True` results in a conversion of the data-values
+            to floats!! For very large datasets this can cause a huge increase in
+            memory-usage! EOmaps handles the scaling internally to show correct
+            values for callbacks and colorbars, even if `mask_and_scale=False`!
+
+            The default is False.
 
         Returns
         -------
@@ -109,10 +134,12 @@ class read_file:
 
         opened = False  # just an indicator if we have to close the file in the end
         try:
-            if isinstance(path_or_dataset, str):
+            if isinstance(path_or_dataset, (str, Path)):
                 # if a path is provided, open the file (and close it in the end)
                 opened = True
-                ncfile = xar.open_dataset(path_or_dataset)
+                ncfile = xar.open_dataset(
+                    path_or_dataset, mask_and_scale=mask_and_scale
+                )
             elif isinstance(path_or_dataset, xar.Dataset):
                 # if an xar.Dataset is provided, use it
                 ncfile = path_or_dataset
@@ -125,7 +152,6 @@ class read_file:
                 usencfile = ncfile
 
             ncdims = list(usencfile.dims)  # dimension order as stored in the file
-
             varnames = list(usencfile)
             if len(varnames) > 1:
                 raise AssertionError(
@@ -160,15 +186,38 @@ class read_file:
             # (e.g. if data is provided with [y, x] dimensions instead of [x, y])
             data = np.moveaxis(usencfile.values, *[dims.index(i) for i in ncdims])
 
-            xcoord, ycoord = (
+            x, y = (
                 getattr(usencfile, ncdims[0]).values,
                 getattr(usencfile, ncdims[1]).values,
             )
 
-            if set_data is not None:
-                set_data.set_data(data=data, xcoord=xcoord, ycoord=ycoord, crs=data_crs)
+            # only use masked arrays if mask_and_scale is False!
+            # (otherwise the mask is already applied as NaN's in the float-array)
+            # Using masked-arrays ensures that we can deal with integers as well!
+            if mask_and_scale is False:
+                encoding = usencfile.attrs
+                fill_value = encoding.get("_FillValue", None)
+                if fill_value:
+                    data = np.ma.masked_where(data == fill_value, data, copy=False)
             else:
-                return dict(data=data, xcoord=xcoord, ycoord=ycoord, crs=data_crs)
+                encoding = None
+
+            if set_data is not None:
+                set_data.set_data(
+                    data=data,
+                    x=x,
+                    y=y,
+                    crs=data_crs,
+                    encoding=encoding,
+                )
+            else:
+                return dict(
+                    data=data,
+                    x=x,
+                    y=y,
+                    crs=data_crs,
+                    encoding=encoding,
+                )
         finally:
             if opened:
                 ncfile.close()
@@ -183,6 +232,7 @@ class read_file:
         sel=None,
         isel=None,
         set_data=None,
+        mask_and_scale=False,
     ):
         """
         Read all relevant information necessary to add a NetCDF to the map.
@@ -224,6 +274,24 @@ class read_file:
             Indicator if the dataset should be returned None or assigned to the
             provided Maps-object  (e.g. by using m.set_data(...)).
             The default is None.
+        mask_and_scale : bool
+            Indicator if the data should be masked and scaled with respect to the
+            file-attributes *_FillValue*, *scale_factor* and *add_offset*.
+
+            - If False: the data will only be scaled "on demand", avoiding costly
+              initialization of very large float-arrays. Masking will still be applied
+              in case a *_FillValue* attribute has been found.
+              (a masked-array is returned to avoid dtype conversions)
+              The encoding is accessible via `m.data_specs.encoding`
+            - If True: all data will be masked and scaled after reading.
+              For more details see `xarray.open_dataset`.
+
+            NOTE: using `mask_and_scale=True` results in a conversion of the data-values
+            to floats!! For very large datasets this can cause a huge increase in
+            memory-usage! EOmaps handles the scaling internally to show correct
+            values for callbacks and colorbars, even if `mask_and_scale=False`!
+
+            The default is False.
 
         Returns
         -------
@@ -258,7 +326,7 @@ class read_file:
 
         opened = False  # just an indicator if we have to close the file in the end
         try:
-            if isinstance(path_or_dataset, str):
+            if isinstance(path_or_dataset, (str, Path)):
                 # if a path is provided, open the file (and close it in the end)
                 opened = True
                 ncfile = xar.open_dataset(path_or_dataset)
@@ -308,9 +376,9 @@ class read_file:
             )
 
             if coords[0] in usencfile.coords:
-                xcoord = usencfile.coords[coords[0]]
+                x = usencfile.coords[coords[0]]
             elif coords[0] in usencfile:
-                xcoord = usencfile[coords[0]]
+                x = usencfile[coords[0]]
             else:
                 raise AssertionError(
                     f"EOmaps: Coordinate '{coords[0]}' is not present in the NetCDF.\n"
@@ -319,9 +387,9 @@ class read_file:
                 )
 
             if coords[1] in usencfile.coords:
-                ycoord = usencfile.coords[coords[1]]
+                y = usencfile.coords[coords[1]]
             elif coords[1] in usencfile:
-                ycoord = usencfile[coords[1]]
+                y = usencfile[coords[1]]
             else:
                 raise AssertionError(
                     f"EOmaps: Coordinate '{coords[1]}' is not present in the NetCDF\n"
@@ -329,31 +397,52 @@ class read_file:
                     + f"Available variables: {list(ncfile)}"
                 )
 
-            if data.dims != xcoord.dims or data.dims != xcoord.dims:
-                raise AssertionError(
-                    "EOmaps: Invalid dimensions of data and coordinates!\n"
-                    + f"data: {data.dims},  xcoord: {xcoord.dims}, ycoord: {ycoord.dims}"
-                )
+            # 1D coordinates + 2D data expects transposed data!
+            transpose = False
+            if data.dims != x.dims or data.dims != x.dims:
+                if (len(data.dims) == 2 and len(x.dims) == 1 and len(y.dims) == 1) and (
+                    data.dims[0] == x.dims[0] and data.dims[1] == y.dims[0]
+                ):
+                    x, y = y, x
+                    transpose = True
+                elif (
+                    len(data.dims) == 2 and len(x.dims) == 1 and len(y.dims) == 1
+                ) and (data.dims[0] == y.dims[0] and data.dims[1] == x.dims[0]):
+                    transpose = True
+                else:
+                    raise AssertionError(
+                        "EOmaps: Invalid dimensions of data and coordinates!\n"
+                        + f"data: {data.dims},  x: {x.dims}, y: {y.dims}"
+                    )
 
-            # xcoord, ycoord = np.meshgrid(
-            #     usencfile.coords[coords[0]].values, usencfile.coords[coords[1]].values
-            # )
+            # only use masked arrays if mask_and_scale is False!
+            # (otherwise the mask is already applied as NaN's in the float-array)
+            # Using masked-arrays ensures that we can deal with integers as well!
+            if mask_and_scale is False:
+                encoding = usencfile.attrs
+                fill_value = encoding.get("_FillValue", None)
+                if fill_value:
+                    data = np.ma.masked_where(data == fill_value, data, copy=False)
+            else:
+                encoding = None
 
             if set_data is not None:
                 set_data.set_data(
-                    data=data.values,
-                    xcoord=xcoord.values,
-                    ycoord=ycoord.values,
+                    data=data.values.T if transpose else data.values,
+                    x=x.values,
+                    y=y.values,
                     crs=data_crs,
                     parameter=parameter,
+                    encoding=encoding,
                 )
             else:
                 return dict(
-                    data=data.values,
-                    xcoord=xcoord.values,
-                    ycoord=ycoord.values,
+                    data=data.values.T if transpose else data.values,
+                    x=x.values,
+                    y=y.values,
                     crs=data_crs,
                     parameter=parameter,
+                    encoding=encoding,
                 )
 
         finally:
@@ -364,8 +453,8 @@ class read_file:
     def CSV(
         path,
         parameter=None,
-        xcoord=None,
-        ycoord=None,
+        x=None,
+        y=None,
         crs=None,
         set_data=None,
         **kwargs,
@@ -387,10 +476,10 @@ class read_file:
             The path to the csv-file.
         parameter : str
             The column-name to use as parameter.
-        xcoord : str
-            The column-name to use as xcoord.
-        ycoord : str
-            The column-name to use as ycoord.
+        x : str
+            The column-name to use as "x" coordinates.
+        y : str
+            The column-name to use as "y" coordinates.
         crs : crs-identifier
             The crs of the data. (see "Maps.set_data" for details)
 
@@ -410,7 +499,7 @@ class read_file:
 
         data = pd.read_csv(path, **kwargs)
 
-        for key in [parameter, xcoord, ycoord]:
+        for key in [parameter, x, y]:
             assert key in data, (
                 f"EOmaps: the parameter-name {key} is not a column of the csv-file!\n"
                 + f"Available columns are: {list(data)}"
@@ -418,17 +507,17 @@ class read_file:
 
         if set_data is not None:
             set_data.set_data(
-                data=data[[parameter, xcoord, ycoord]],
-                xcoord=xcoord,
-                ycoord=ycoord,
+                data=data[[parameter, x, y]],
+                x=x,
+                y=y,
                 crs=crs,
                 parameter=parameter,
             )
         else:
             return dict(
-                data=data[[parameter, xcoord, ycoord]],
-                xcoord=xcoord,
-                ycoord=ycoord,
+                data=data[[parameter, x, y]],
+                x=x,
+                y=y,
                 crs=crs,
                 parameter=parameter,
             )
@@ -438,13 +527,12 @@ def _from_file(
     data,
     crs=None,
     shape=None,
-    plot_specs=None,
     classify_specs=None,
     val_transform=None,
     coastline=True,
     parent=None,
     figsize=None,
-    layer=0,
+    layer=None,
     **kwargs,
 ):
     """
@@ -461,9 +549,8 @@ def _from_file(
 
         >>> m = Maps(crs=..., layer=...)
         >>> m.set_data(**m.read_GeoTIFF(...))
-        >>> m.set_plot_specs(...)
         >>> m.set_classify_specs(...)
-        >>> m.plot_map(...)
+        >>> m.plot_map(**kwargs)
 
     Parameters
     ----------
@@ -479,9 +566,6 @@ def _from_file(
     isel : dict, optional
         A dict of keyword-arguments passed to `xarray.Dataset.isel()`.
         The default is None.
-    plot_specs : dict, optional
-        A dict of keyword-arguments passed to `m.set_plot_specs()`.
-        The default is None.
     classify_specs : dict, optional
         A dict of keyword-arguments passed to `m.set_classify_specs()`.
         The default is None.
@@ -513,14 +597,18 @@ def _from_file(
         data["data"] = val_transform(data["data"])
 
     if parent is not None:
+        if layer is None:
+            layer = parent.layer
+
         m = parent.new_layer(
             copy_data_specs=False,
-            copy_plot_specs=False,
             copy_classify_specs=False,
             copy_shape=False,
             layer=layer,
         )
     else:
+        if layer is None:
+            layer = 0
         # get crs from file
         if crs is None:
             crs = data.get("crs", None)
@@ -540,14 +628,7 @@ def _from_file(
     if coastline:
         m.add_feature.preset.coastline()
 
-    kwkeys = list(kwargs)
-    for key in kwkeys:
-        if key in m.plot_specs.keys():
-            m.plot_specs[key] = kwargs.pop(key)
-
     m.set_data(**data)
-    if plot_specs:
-        m.set_plot_specs(**plot_specs)
     if classify_specs:
         m.set_classify_specs(**classify_specs)
 
@@ -558,34 +639,7 @@ def _from_file(
         elif isinstance(shape, dict):
             getattr(m.set_shape, shape.pop("shape"))(**shape)
 
-        m.plot_map(**kwargs)
-        return m
-
-    else:
-        # only try to plot as raster if in_crs == out_crs
-        crs1 = CRS.from_user_input(m.data_specs.crs)
-        crs2 = CRS.from_user_input(m._crs_plot)
-        if crs1.equals(crs2):
-            try:
-                # try to plot as raster - shading...
-                m.set_shape.shade_raster()
-                m.plot_map(**kwargs)
-                return m
-            except Exception:
-                pass
-
-        # try to plot as point - shading...
-        try:
-            m.set_shape.shade_points()
-            m.plot_map(**kwargs)
-            return m
-        except Exception:
-            pass
-
-        # try to plot as ellipses
-        m.set_shape.ellipses()
-        m.plot_map(**kwargs)
-
+    m.plot_map(**kwargs)
     return m
 
 
@@ -612,10 +666,10 @@ class from_file:
         isel=None,
         plot_crs=None,
         shape=None,
-        plot_specs=None,
         classify_specs=None,
         val_transform=None,
         coastline=True,
+        mask_and_scale=False,
         **kwargs,
     ):
         """
@@ -632,10 +686,8 @@ class from_file:
 
         >>> m = Maps(crs=...)
         >>> m.set_data(**m.read_file.NetCDF(...))
-        >>> m.set_plot_specs(...)
         >>> m.set_classify_specs(...)
-        >>> m.plot_map(...)
-
+        >>> m.plot_map(**kwargs)
 
         Parameters
         ----------
@@ -679,16 +731,13 @@ class from_file:
             possible, else epsg=4326.
         shape : str, dict or None, optional
             - if str: The name of the shape to use, e.g. one of:
-              ['geod_circles', 'ellipses', 'rectangles', 'voroni_diagram',
+              ['geod_circles', 'ellipses', 'rectangles', 'voronoi_diagram',
               'delaunay_triangulation', 'shade_points', 'shade_raster']
             - if dict: a dictionary with parameters passed to the selected shape.
               The dict MUST contain a key "shape" that holds the name of the shape!
 
               >>> dict(shape="rectangles", radius=1, radius_crs=.5)
 
-        plot_specs : dict, optional
-            A dict of keyword-arguments passed to `m.set_plot_specs()`.
-            The default is None.
         classify_specs : dict, optional
             A dict of keyword-arguments passed to `m.set_classify_specs()`.
             The default is None.
@@ -701,6 +750,24 @@ class from_file:
         coastline: bool
             Indicator if a coastline should be added or not.
             The default is True
+        mask_and_scale : bool
+            Indicator if the data should be masked and scaled with respect to the
+            file-attributes *_FillValue*, *scale_factor* and *add_offset*.
+
+            - If False: the data will only be scaled "on demand", avoiding costly
+              initialization of very large float-arrays. Masking will still be applied
+              in case a *_FillValue* attribute has been found.
+              (a masked-array is returned to avoid dtype conversions)
+              The encoding is accessible via `m.data_specs.encoding`
+            - If True: all data will be masked and scaled after reading.
+              For more details see `xarray.open_dataset`.
+
+            NOTE: using `mask_and_scale=True` results in a conversion of the data-values
+            to floats!! For very large datasets this can cause a huge increase in
+            memory-usage! EOmaps handles the scaling internally to show correct
+            values for callbacks and colorbars, even if `mask_and_scale=False`!
+
+            The default is False.
         kwargs :
             Keyword-arguments passed to `m.plot_map()`
 
@@ -750,6 +817,7 @@ class from_file:
             sel=sel,
             isel=isel,
             set_data=None,
+            mask_and_scale=mask_and_scale,
         )
 
         if val_transform:
@@ -759,7 +827,6 @@ class from_file:
             data,
             crs=plot_crs,
             shape=shape,
-            plot_specs=plot_specs,
             classify_specs=classify_specs,
             val_transform=val_transform,
             coastline=coastline,
@@ -775,10 +842,10 @@ class from_file:
         isel=None,
         plot_crs=None,
         shape=None,
-        plot_specs=None,
         classify_specs=None,
         val_transform=None,
         coastline=True,
+        mask_and_scale=False,
         **kwargs,
     ):
         """
@@ -795,9 +862,8 @@ class from_file:
 
         >>> m = Maps(crs=...)
         >>> m.set_data(**m.read_file.GeoTIFF(...))
-        >>> m.set_plot_specs(...)
         >>> m.set_classify_specs(...)
-        >>> m.plot_map(...)
+        >>> m.plot_map(**kwargs)
 
         Parameters
         ----------
@@ -829,16 +895,13 @@ class from_file:
             possible, else epsg=4326.
         shape : str, dict or None, optional
             - if str: The name of the shape to use, e.g. one of:
-              ['geod_circles', 'ellipses', 'rectangles', 'voroni_diagram',
+              ['geod_circles', 'ellipses', 'rectangles', 'voronoi_diagram',
               'delaunay_triangulation', 'shade_points', 'shade_raster']
             - if dict: a dictionary with parameters passed to the selected shape.
               The dict MUST contain a key "shape" that holds the name of the shape!
 
               >>> dict(shape="rectangles", radius=1, radius_crs=.5)
 
-        plot_specs : dict, optional
-            A dict of keyword-arguments passed to `m.set_plot_specs()`.
-            The default is None.
         classify_specs : dict, optional
             A dict of keyword-arguments passed to `m.set_classify_specs()`.
             The default is None.
@@ -851,6 +914,24 @@ class from_file:
         coastline: bool
             Indicator if a coastline should be added or not.
             The default is True
+        mask_and_scale : bool
+            Indicator if the data should be masked and scaled with respect to the
+            file-attributes *_FillValue*, *scale_factor* and *add_offset*.
+
+            - If False: the data will only be scaled "on demand", avoiding costly
+              initialization of very large float-arrays. Masking will still be applied
+              in case a *_FillValue* attribute has been found.
+              (a masked-array is returned to avoid dtype conversions)
+              The encoding is accessible via `m.data_specs.encoding`
+            - If True: all data will be masked and scaled after reading.
+              For more details see `xarray.open_dataset`.
+
+            NOTE: using `mask_and_scale=True` results in a conversion of the data-values
+            to floats!! For very large datasets this can cause a huge increase in
+            memory-usage! EOmaps handles the scaling internally to show correct
+            values for callbacks and colorbars, even if `mask_and_scale=False`!
+
+            The default is False.
         kwargs :
             Keyword-arguments passed to `m.plot_map()`
 
@@ -896,6 +977,7 @@ class from_file:
             set_data=None,
             data_crs=data_crs,
             crs_key=data_crs_key,
+            mask_and_scale=mask_and_scale,
         )
 
         if val_transform:
@@ -905,7 +987,6 @@ class from_file:
             data,
             crs=plot_crs,
             shape=shape,
-            plot_specs=plot_specs,
             classify_specs=classify_specs,
             val_transform=val_transform,
             coastline=coastline,
@@ -916,12 +997,11 @@ class from_file:
     def CSV(
         path=None,
         parameter=None,
-        xcoord=None,
-        ycoord=None,
+        x=None,
+        y=None,
         data_crs=None,
         plot_crs=None,
         shape=None,
-        plot_specs=None,
         classify_specs=None,
         val_transform=None,
         coastline=True,
@@ -941,9 +1021,8 @@ class from_file:
 
         >>> m = Maps(crs=...)
         >>> m.set_data(**m.read_file.CSV(...))
-        >>> m.set_plot_specs(...)
         >>> m.set_classify_specs(...)
-        >>> m.plot_map(...)
+        >>> m.plot_map(**kwargs)
 
 
         Parameters
@@ -952,10 +1031,10 @@ class from_file:
             The path to the csv-file.
         parameter : str
             The column-name to use as parameter.
-        xcoord : str
-            The column-name to use as xcoord.
-        ycoord : str
-            The column-name to use as ycoord.
+        x : str
+            The column-name to use as "x" coordinate.
+        y : str
+            The column-name to use as "y" coordinate.
         data_crs : crs-identifier
             The crs of the data. (see "Maps.set_data" for details)
         plot_crs : any, optional
@@ -964,16 +1043,13 @@ class from_file:
             possible, else epsg=4326.
         shape : str, dict or None, optional
             - if str: The name of the shape to use, e.g. one of:
-              ['geod_circles', 'ellipses', 'rectangles', 'voroni_diagram',
+              ['geod_circles', 'ellipses', 'rectangles', 'voronoi_diagram',
               'delaunay_triangulation', 'shade_points', 'shade_raster']
             - if dict: a dictionary with parameters passed to the selected shape.
               The dict MUST contain a key "shape" that holds the name of the shape!
 
               >>> dict(shape="rectangles", radius=1, radius_crs=.5)
 
-        plot_specs : dict, optional
-            A dict of keyword-arguments passed to `m.set_plot_specs()`.
-            The default is None.
         classify_specs : dict, optional
             A dict of keyword-arguments passed to `m.set_classify_specs()`.
             The default is None.
@@ -1000,8 +1076,8 @@ class from_file:
         data = read_file.CSV(
             path=path,
             parameter=parameter,
-            xcoord=xcoord,
-            ycoord=ycoord,
+            x=x,
+            y=y,
             crs=data_crs,
         )
 
@@ -1009,7 +1085,6 @@ class from_file:
             data,
             crs=plot_crs,
             shape=shape,
-            plot_specs=plot_specs,
             classify_specs=classify_specs,
             val_transform=val_transform,
             coastline=coastline,
